#ifndef SIMPLE_MODBUS_MASTER_H
#define SIMPLE_MODBUS_MASTER_H
/* 
  SimpleModbusMasterV2r2
  SimpleModbusMaster   
    slave ,   RTU Modbus.

     slave ,    ,    
    , ,    slave .
        .
    ,    . 
            .

       modbus:    

   requests -    slave .
   successful_requests -    .
   failed_requests -   ,      . 
   retries -   .
   exception_errors -   modbus   .
      ,  ,   
      .

       ""   
          
    .     
   .      
     ,     
      .   
   - ,   modbus .
     slave ,    , 
        . .
      1500 ,     10 slave   9  
   ,   ,   
    1500  * 9 = 13,5 !  
        ,     
      .

     ,    
      .

     slave ,  modbus
   RTU   ,  
   slave ,  ,  
     ,  .
     1, 2, 3, 4, 5, 6, 15  16. 
       ( = 0)    5, 6, 15  16.

   :
   1 -   READ_COIL_STATUS
   2 -   READ_INPUT_STATUS               -      .
   3 -   READ_HOLDING_REGISTERS      -      .
   4 -   READ_INPUT_REGISTERS          -      .
   5 -   FORCE_SINGLE_COIL               -    .
   6 -   PRESET_SINGLE_REGISTER       -      .
   15 - FORCE_MULTIPLE_COILS          -      .
   16 - PRESET_MULTIPLE_REGISTERS  -      

:  
        64   32 .
       RS-485   MAX485  .

    3  4  master       slave 
      5     ID, FUNCTION, NO OF BYTES   BYTES CRC 
   master      58   29 .

    16  master       slave    9  
      ID, FUNCTION, ADDRESS, NO OF REGISTERS, NO OF BYTES   BYTES CRC 
         54   27 .

   :
      USB / Serial     ,    , 
      ,      . 

     ,       ,    USB / Serial 
            ,
     64 .
*/

#include "Arduino.h"

#define COIL_OFF 0x0000                      //  5 OFF  0x0000
#define COIL_ON 0xFF00                       //  5 ON   0xFF00
#define READ_COIL_STATUS             1    //   ON/OFF   (0X references, coils)  slave .
#define READ_INPUT_STATUS           2    //   ON/OFF   (1X references)  slave .
#define READ_HOLDING_REGISTERS  3    //        (4X references)  slave .
#define READ_INPUT_REGISTERS      4    //      (3X references)  slave .  .
#define FORCE_SINGLE_COIL           5    //  single  (0X reference) - ON (0xFF00)  OFF (0x0000).
#define PRESET_SINGLE_REGISTER   6    //         (4X reference).
#define FORCE_MULTIPLE_COILS         15  //   (0X reference)    ON  OFF.
#define PRESET_MULTIPLE_REGISTERS 16  //          (4X references).

typedef struct
{
  //   
  unsigned char id;
  unsigned char function;
  unsigned int address;
	
  //   1  2  -  
  //   5   ON (oxFF00)  OFF (0x0000)
  //   6   ,   
  //   3, 4  16  -  
  //   15   

  unsigned int data; 	
  unsigned int local_start_address;
  
  // modbus  .
  unsigned int requests;
  unsigned int successful_requests;
  unsigned int failed_requests;
  unsigned int exception_errors;
  unsigned int retries;
  	
  //   
  unsigned char connection; 
  
}Packet;

//  
void modbus_update();

void modbus_construct(Packet *_packet, 
unsigned char id, 
unsigned char function, 
unsigned int address, 
unsigned int data,
unsigned _local_start_address);
											
void modbus_configure(HardwareSerial* SerialPort,
long baud, 
unsigned char byteFormat,
long _timeout, 
long _polling, 
unsigned char _retry_count, 
unsigned char _TxEnablePin,
Packet* _packets, 
unsigned int _total_no_of_packets,
unsigned int* _register_array);
#endif