/*
   The example will use packet1 to read a register from address 0 (the adc ch0 value)
   from the arduino slave (id=1). It will then use this value to adjust the brightness
   of an led on pin 9 using PWM.
   It will then use packet2 to write a register (its own adc ch0 value) to address 1 
   on the arduino slave (id=1) adjusting the brightness of an led on pin 9 using PWM.
*/
#include <SimpleModbusMaster.h>
//////////////////// Port information ///////////////////
#define baud        9600
#define timeout     1000
#define polling     200 // скорость опроса по модбус
#define retry_count 10
#define TxEnablePin 2   // Tx/Rx пин RS485
#define LED1        9   // светодиод 1
#define LED2        13  // светодиод 2
//Общая сумма доступной памяти на ведущем устройстве, чтобы хранить данные
#define TOTAL_NO_OF_REGISTERS 4

// This is the easiest way to create new packets
// Add as many as you want. TOTAL_NO_OF_PACKETS
// is automatically updated.
enum
{
  PACKET1,
  PACKET2,
  PACKET3,
  PACKET4,
  TOTAL_NO_OF_PACKETS // leave this last entry
};

// Масив пакетов модбус
Packet packets[TOTAL_NO_OF_PACKETS];

// Массив хранения содержимого принятых и передающихся регистров
unsigned int regs[TOTAL_NO_OF_REGISTERS];

void setup()
{
  // Настраиваем пакеты
  // Шестой параметр - это индекс ячейки в массиве, размещенном в памяти ведущего устройства, в которую будет 
  // помещен результат или из которой будут браться данные для передачи в подчиненное устройство. В нашем коде - это массив reg
  // Пакет,SLAVE адрес,функция модбус,адрес регистра,количесво запрашиваемых регистров,локальный адрес регистра.
  modbus_construct(&packets[PACKET1], 1, READ_HOLDING_REGISTERS,    0, 1, 0);
  modbus_construct(&packets[PACKET2], 1, READ_HOLDING_REGISTERS,    1, 1, 0);
  // Пакет,SLAVE адрес,функция модбус,адрес регистра,данные,локальный адрес регистра.
  modbus_construct(&packets[PACKET3], 1, PRESET_MULTIPLE_REGISTERS, 2, 1, 0); 
  modbus_construct(&packets[PACKET4], 1, PRESET_MULTIPLE_REGISTERS, 3, 1, 0);
  
  // инициализируем протокол модбус
  modbus_configure(&Serial, baud, SERIAL_8N1, timeout, polling, retry_count, TxEnablePin, packets, TOTAL_NO_OF_PACKETS, regs);
  
  pinMode(LED1, OUTPUT);
  pinMode(LED2, OUTPUT);
}

void loop()
{
  modbus_update();
  unsigned int temp1;
  unsigned int temp2;
  //regs[0] = analogRead(0);       // запись данных master-slave
  //analogWrite(LED, regs[0]>>2);  // чтение данных slave-master (ограничеть количесво бит данных числом 255)
  regs[0] = 255;   // запись данных master-slave (slave адрес 1, регистр 1)
  regs[1] = 255;   // запись данных master-slave (slave адрес 1, регистр 2) 
  temp1 = regs[2]; // чтение данных slave-master (slave адрес 1, регистр 3)  
  temp2 = regs[3]; // чтение данных slave-master (slave адрес 1, регистр 4) 
   if (temp1 == 255){digitalWrite(LED1, HIGH);}else{digitalWrite(LED1, LOW);}//если пришло 255 зажигаем светодиод
   if (temp2 == 255){digitalWrite(LED2, HIGH);}else{digitalWrite(LED2, LOW);}
}
